import json
from urllib.parse import urlencode

import aiohttp
from aiohttp.client import ClientResponse


def build_url_with_params(url: str, key: str, params=None) -> str:
    if params is None:
        params = {}
    encoded = urlencode(clean_dict(params))
    return url + "?key=" + key if len(encoded) == 0 else (url + "?key=" + key + "&" + encoded)


def build_url_with_params_for_search(url: str, search: str, params=None) -> str:
    if params is None:
        params = {}
    encoded = urlencode(clean_dict(params))
    return url + "?term=" + search if (len(encoded) == 0) else (url + "?term=" + search + "&" + encoded)


def clean_dict(x=None) -> dict:
    if x is None:
        x = {}
    result = {}
    for key in x:
        if x[key]:
            # Check If List
            if isinstance(x[key], list):
                result[key] = ",".join(x[key])
            # Check If Boolean
            elif isinstance(x[key], bool):
                if x[key] is True:
                    result[key] = "true"
                else:
                    result[key] = "false"
            # Everything Else (Strings/Numbers)
            else:
                result[key] = x[key]
    return result


def merge_dict(x: dict, y: dict) -> dict:
    z = clean_dict(x)
    z.update(clean_dict(y))
    return z


async def validator(result: ClientResponse) -> str or dict:
    try:
        body = await result.json()
    except aiohttp.ContentTypeError:
        body = await result.text()
    except Exception as e:
        print(e)
        body = {}

    if isinstance(body, dict) and body.get("code"):
        raise Exception(body.get("description"))
    elif result.status >= 400:
        raise Exception(f"{result.status}, {result.reason}")
    elif not body:
        return "OK"
    else:
        return body


def retry(times, exceptions):
    """
    Retry Decorator
    Retries the wrapped function/method `times` times if the exceptions listed
    in ``exceptions`` are thrown
    :param times: The number of times to repeat the wrapped function/method
    :type times: Int
    :param exceptions: Lists of exceptions that trigger a retry attempt
    :type exceptions: Tuple of Exceptions
    """

    def decorator(func):
        async def new_fn(*args, **kwargs):
            for attempt in range(times):
                try:
                    return await func(*args, **kwargs)
                except exceptions:
                    print(f'Exception thrown when attempting to run {func.__name__}, attempt {attempt + 1} of {times}')
            return func(*args, **kwargs)

        return new_fn

    return decorator


def create_session(fn):
    async def wrapper(*args, **kwargs):
        async with aiohttp.ClientSession() as session:
            return await fn(*args, session=session, **kwargs)

    return wrapper



import json

from aiohttp import ClientSession

from .constants import API_BASE_URL
from .utils import build_url_with_params, merge_dict, retry, validator, create_session


class Client:
    """Steams API HTTP client"""

    def __init__(self, key: str, headers: dict = None):
        if not headers:
            headers = {}
        """Constructor for TypeForm API client"""
        self.__headers = merge_dict({"Content-Type": "application/json", "Accept": "application/json"}, headers)
        self.key = key

    @retry(times=3, exceptions=(ValueError, TypeError))
    @create_session
    async def request(self, method: str, url: str, data=None, params=None, headers=None, session: ClientSession = None,
                      **kwargs) -> str or dict:

        if headers is None:
            headers = {}
        if params is None:
            params = {}
        if data is None:
            data = {}

        request_url = build_url_with_params((API_BASE_URL + url), self.key, params)

        request_headers = merge_dict(self.__headers, headers)
        request_data = ""
        if type(data) is dict:
            request_data = json.dumps(data) if data.keys() else ""

        if type(data) is list:
            request_data = json.dumps(data) if data else ""
        resp = await session.request(method, request_url, data=request_data, headers=request_headers, **kwargs)
        return await validator(resp)



from .users import Users
from .client import Client
from .apps import Apps


class Steam:
    """Steam API client"""

    def __init__(self, key: str, headers=None):
        """Constructor for Steam API client"""
        if headers is None:
            headers = {}
        client = Client(key, headers=headers)
        self.__users = Users(client)
        self.__apps = Apps(client)

    @property
    def users(self) -> Users:
        return self.__users

    @property
    def apps(self) -> Apps:
        return self.__apps



import json

import typing
from aiohttp import ClientSession
from .client import Client
from .utils import build_url_with_params_for_search, validator, create_session
from bs4 import BeautifulSoup
from .constants import API_APP_DETAILS_URL, API_APP_SEARCH_URL


class Apps:
    """Steam Apps API client"""

    def __init__(self, client: Client):
        """Constructor for Steam Apps class"""
        self.__client = client
        self.__search_url = API_APP_SEARCH_URL
        self.__app_details_url = API_APP_DETAILS_URL

    @create_session
    async def get_app_details(self, app_id: int, country="US",
                              filters: typing.Optional[str] = "basic", session: ClientSession = None) -> dict:
        """Obtains an apps details
        
        Args:
            app_id (int): App ID. For example 546560 (Half-Life-Alyx)
            country (str): ISO Country Code
            session: aiohttp.ClientSession, optional added unfathomably from decorator
            filters (str): list of keys to return, e.g. "name,platforms,price_overview". If you use multiple appids, you must set this parameter to "price_overview".
                The filter basic returns the following keys:
                    type
                    name
                    steam_appid
                    required_age
                    dlc
                    detailed_description
                    about_the_game
                    supported_languages
                    detailed_description
                    supported_languages
                    header_image
                    website
                    pc_requirements
                    mac_requirements
                    linux_requirements
                Any key names except those listed under basic are acceptable as filter values.
                Optional filters:
                    controller_support,
                    fullgame,
                    legal_notice,
                    developers,
                    demos,
                    price_overview,
                    metacritic,
                    categories,
                    genres,
                    screenshots,
                    movies,
                    recommendations,
                    achievements,
        """
        response = await session.request('get', self.__app_details_url,
                                         params={"appids": app_id, "cc": country, "filters": filters})
        json_loaded_response = json.loads(await response.text())
        return json_loaded_response

    async def get_user_stats(self, steam_id: int or str, app_id: int or str) -> dict:
        """Obtains a user's stats for a specific app, includes only completed achievements
        along with app specific information
        
        Args:
            steam_id (int): Steam 64 ID
            app_id (int): App ID
        """
        response = await self.__client.request("get", "/ISteamUserStats/GetUserStatsForGame/v2/",
                                               params={"steamid": steam_id, "appid": app_id})
        return response

    async def get_user_achievements(self, steam_id: int or str, app_id: int or str) -> dict:
        """Obtains information of the user's achievments in the app
        
        Args:
            steam_id (int): Steam 64 ID
            app_id (int): App ID
        """
        response = await self.__client.request("get", "/ISteamUserStats/GetPlayerAchievements/v1/",
                                               params={"steamid": steam_id, "appid": app_id})
        return response

    # Is term meant to be any or a string, I'm not familiar enough with async_steam search so I'll leave it as is
    @create_session
    async def search_games(self, term, country="US", session: ClientSession = None):
        """Searches for games using the information given
        Args:
            term (Any): Search term
            country (str): ISO Country Code
            session: aiohttp.ClientSession, optional added unfathomably from decorator

        """
        url = self.search_url(term, country)
        result = await session.request("get", url)
        html = await validator(result)
        soup = BeautifulSoup(html, features="html.parser")
        links = soup.find_all("a")
        apps = []
        for link in links:
            if link.has_attr("data-ds-appid"):
                app = {}
                string_id = link["data-ds-appid"]
                href = link["href"].replace("\\", "").replace('"', "")
                app["id"] = [int(i) for i in string_id.replace("\\", "").replace('"', "").split(',')]
                app["link"] = href
                divs = link.select("div")
                for div in divs:
                    if div["class"][0] == "match_name":
                        app["name"] = div.text
                    if div["class"][0] == "match_price":
                        app["price"] = div.text
                    if div["class"][0] == "match_img":
                        app["img"] = div.img["src"].replace("\\", "").replace('"', "")
                apps.append(app)
        return {"apps": apps}

    # This should be a private method imo, I don't know how you would like to name them so I'll leave it as is
    # (Maybe change it to all caps since __search_url and __app_details_url are constants?)
    def search_url(self, search, country="US"):
        params = {"f": "games", "cc": country, "realm": 1, "l": "english"}
        result = build_url_with_params_for_search(self.__search_url, search, params=params)
        return result



from .client import Client


class Users:
    """Steam Users API client"""

    def __init__(self, client: Client):
        """Constructor for Steam Users class"""
        self.__client = client

    async def search_user(self, search: str) -> dict or str:
        """Searches for exact match

        Args:
            search (str): async_steam user. For example 'the12thchairman'
        """
        search_response = await self.__client.request("get", "/ISteamUser/ResolveVanityURL/v1/",
                                                      params={"vanityurl": search})

        if search_response["response"]["success"] != 1:
            return search_response["response"]["message"]
        steam_id = search_response["response"]["steamid"]
        return await self.get_user_details(steam_id)

    async def get_user_details(self, steam_id: str or int, single=True) -> dict or str:
        """Gets user/player details by async_steam ID

        Args:
            steam_id (str or int): Steam 64 ID
            single (bool, optional): Gets one player. Defaults to True. When false, steam_id can be a string of steamids and delimited by a ','

        """
        user_response = await self.__client.request("get", "/ISteamUser/GetPlayerSummaries/v2/",
                                                    params={"steamids": steam_id})
        if single:
            result = user_response["response"]["players"]
            return {"player": result[0]} if result else 'No match'
        else:
            return {"players": user_response["response"]["players"]}

    async def get_user_friends_list(self, steam_id: str or int) -> dict:
        """Gets friend list of a user

        Args:
            steam_id (str): Steam 64 ID
        """
        friends_list_response = await self.__client.request("get", "/ISteamUser/GetFriendList/v1/",
                                                            params={"steamid": steam_id})
        transform_friends = await self._transform_friends(friends_list_response["friendslist"])
        return {"friends": transform_friends}

    async def get_user_recently_played_games(self, steam_id: str or int) -> dict:
        """Gets recently played games

        Args:
            steam_id (str): Steam 64 ID
        """
        response = await self.__client.request("get", "/IPlayerService/GetRecentlyPlayedGames/v1/",
                                               params={"steamid": steam_id})
        return response["response"]

    async def get_owned_games(self, steam_id: str or int, include_appinfo=True, includ_free_games=True) -> dict:
        """Gets all owned games of a user by async_steam id

        Args:
            steam_id (str): Steam 64 ID
            include_appinfo (bool, optional): Includes app/game info. Defaults to True.
            includ_free_games (bool, optional): Includes free games. Defaults to True.
        """
        params = {
            "steamid": steam_id,
            "include_appinfo": include_appinfo,
            "include_played_free_games": includ_free_games,
        }
        response = await self.__client.request("get", "/IPlayerService/GetOwnedGames/v1/", params=params)
        return response["response"]

    async def get_user_steam_level(self, steam_id: str or int) -> dict:
        """Gets user async_steam level

        Args:
            steam_id (str): Steam 64 ID
        """
        response = await self.__client.request("get", "/IPlayerService/GetSteamLevel/v1/", params={"steamid": steam_id})
        return response["response"]

    async def get_user_badges(self, steam_id: str or int) -> dict:
        """Gets user async_steam badges

        Args:
            steam_id (str): Steam 64 ID
        """
        response = await self.__client.request("get", "/IPlayerService/GetBadges/v1/", params={"steamid": steam_id})
        return response["response"]

    async def get_community_badge_progress(self, steam_id: str or str, badge_id: int or str) -> dict:
        """Gets user community badge progress

        Args:
            steam_id (str): Steam 64 ID
            badge_id (int): Badge ID
        """
        response = await self.__client.request("get", "/IPlayerService/GetCommunityBadgeProgress/v1",
                                               params={"steamid": steam_id, "badgeid": badge_id}, )
        return response["response"]

    async def get_account_public_info(self, steam_id: str) -> dict:
        """Gets account public info

        Args:
            steam_id (str): Steam 64 ID
        """
        response = await self.__client.request("get", "/IGameServersService/GetAccountPublicInfo/v1",
                                               params={"steamid": steam_id})
        return response

    async def get_player_bans(self, steam_id: str or str) -> dict:
        """Gets account bans info

        Args:
            steam_id (int or str): Steam 64 ID
        """
        response = await self.__client.request("get", "/ISteamUser/GetPlayerBans/v1", params={"steamids": steam_id})
        return response

    async def _transform_friends(self, friends_list: dict) -> dict:
        friend_steam_ids = [friend["steamid"] for friend in friends_list["friends"]]
        friends = await self.get_user_details(",".join(friend_steam_ids), False)
        friends = friends["players"]
        for f in friends:
            found = next(
                item
                for item in friends_list["friends"]
                if item["steamid"] == f["steamid"]
            )
            f["relationship"] = found["relationship"]
            f["friend_since"] = found["friend_since"]

        return friends

    async def get_steamid(self, vanity: str) -> dict:
        """Get steamid64 from vanity URL

        Args:
            vanity (str): Vanity URL
        """
        response = await self.__client.request("get", "/ISteamUser/ResolveVanityURL/v1", params={"vanityurl": vanity})
        return response["response"]



from setuptools import setup, find_packages

with open("README.md") as fp:
    long_description = fp.read()

setup(
    name="aiosteam-api",
    version="1.0.1",
    description="Async Python Client wrapper for Steam API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords=[
        "async steam",
        "steam api",
        "async steam community",
        "api",
        "async"
    ],
    author="Yevhenii Havrus",
    author_email="jeygavrus@gmail.com",
    url="https://github.com/jgavrus/aiosteam-api",
    packages=find_packages(),
    install_requires=["beautifulsoup4", "aiohttp"],
    license="MIT",
    # https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python",
    ],
)
